/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

package org.sf.charttaglibs;

import java.awt.Color;
import java.io.IOException;
import java.io.PrintWriter;
import java.lang.reflect.InvocationTargetException;
import java.util.Arrays;
import java.util.Collection;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.jsp.JspException;

import org.apache.commons.beanutils.BeanUtils;

import org.jfree.chart.ChartRenderingInfo;
import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.entity.StandardEntityCollection;
import org.jfree.chart.servlet.ServletUtilities;
import org.jfree.chart.title.TextTitle;
import org.jfree.ui.RectangleEdge;

import org.sf.charttaglibs.utils.IteratorAdapter;
import org.sf.charttaglibs.utils.TagUtils;

/**
 * The base class that manage all chart-tags
 * it's handle the common and default value for a JFreeChart graphics.
 * All tags calsses must extends this class
 * @author mmohamed |at| dcc |dot| uchile |dot| cl
 *
 */
public abstract class BaseChartTag extends BaseHandlerTag {
	
	/// The chart to draw
	protected JFreeChart chart = null;
	/// The URL SRC to draw
	protected String src = null;
	
	/// JFreeChart attribute
	
	/// The title of the chart
	protected String title = null;
	/// The subtitle of the chart
	protected String subtitle = null;
	/// The chart border ,ie, the border that JFreeChart will draw, this attribute
	/// is independent of the html border element
	protected boolean chartborder = false;
	/// The chart will have a MAP
	protected boolean usemap = false;
	/// The antialias is on or off
	protected boolean antiAlias = false;
	/// Display the legend?
	protected boolean createLegend = true;
	/// Create tooltip?
	protected boolean createTooltip = true;
	/// Create url?
	protected boolean createUrl = false;
	
	// The default bgColor es white
	protected String bgColor = "white";
	
	/// The title position on the chart
	protected String titlePosition = Constant.DEFAULT_TITLE_POSITION; /// The default title position
	/// The subtitle position
	protected String subtitlePosition = Constant.DEFAULT_SUBTITLE_POSITION; /// The default Subtitle position
	
	/// The forground alpha value
	protected float fgAlpha = -1;
	/// The background alpha value
	protected float bgAlpha = -1;
	
	/// Chart width
	protected int chartWidth = Constant.DEFAULT_CHART_WIDTH;
	/// Chart hight
	protected int chartHight = Constant.DEFAULT_CHART_HIGHT;
	
	
	/// Internal use only
	
	/// The name of the generated file
	private String filename = null;
	
	/*
	 * That abstract method is used to initialze the chart with type and dataset
	 */
	public abstract void initChart() throws JspException;
	
	/**
	 * Execute the logic of the chart and save it to the server temporaly directory
	 * it'l return a null value if the char is null, otherwise it will the
	 * name of the generated file with image
	 * @return filename
	 * @throws JspException if something go wrong
	 */
	public String getFileName() throws JspException {		
		String filename = null;
		ChartRenderingInfo info = null;
		String imgUrl = null;
		try{
			if (chart != null) {
				if(usemap)
					info = new ChartRenderingInfo(new StandardEntityCollection());
				
				// Write the chart image to the temporary directory
				filename = ServletUtilities.saveChartAsPNG(chart, chartWidth, chartHight, 
														info, pageContext.getSession());
				//  Write the image map to the PrintWriter
				PrintWriter pw = new PrintWriter(pageContext.getOut());
				if(usemap) {
					// TODO make sure that the parametr true of
					// ChartUtilities.writeImageMap is ok
					ChartUtilities.writeImageMap(pw, filename, info,true);
					pw.flush();
				}
			}
		} catch (IOException e) {
			throw new JspException(messages.getMessage("write.io"),e);
		}
		return filename;
	}
	
	/**
	 * This method is invoked by the JSP container to parse the tag.
	 * All common task are here
	 * @see javax.servlet.jsp.tagext.Tag#doStartTag()
	 */
	public int doStartTag() throws JspException {
		// Getting the dataset for the chart
		initChart();
		// Setting chart attribute
		chart.setBorderVisible(chartborder);
		chart.setBackgroundPaint(getColorByString(bgColor));
		if(title != null && title.length() > 0)
			setTitlePosition();
		chart.setAntiAlias(antiAlias);
		if(subtitle != null && subtitle.length() > 0) {
			chart.addSubtitle(new TextTitle(subtitle));
			setSubtitlePosition();
		}
		if(getFgAlpha() != -1)
			chart.getPlot().setForegroundAlpha(getFgAlpha());
		if(getBgAlpha() != -1)
			chart.getPlot().setBackgroundAlpha(getBgAlpha());
		
		// save file to server
		filename = getFileName();
		// render the IMG element
		renderImgTag();
		return SKIP_BODY;
	}

	public int doEndTag() throws JspException {
		return EVAL_PAGE;
	}

	/**
	 * Render the tag IMG of the html standard
	 * @throws JspException
	 */
	private void renderImgTag() throws JspException {
		// Preparing the render of the IMG tag (html) 
		HttpServletResponse response = (HttpServletResponse) pageContext.getResponse();
		StringBuffer results = new StringBuffer("<img");
		src = ((HttpServletRequest)pageContext.getRequest()).getContextPath() + Constant.SERVLET_URL + filename;
		prepareAttribute(results, "src", response.encodeURL(src));
		//prepareAttribute(results, "usemap", new Boolean(getUsemap()));
		results.append(prepareHtmlElements());
		results.append(prepareJavaScriptsEvent());
		prepareOtherAttributes(results);
		results.append(getElementClose());
		
		TagUtils.getInstance().write(pageContext, results.toString());
	}
	
	/**
	 * Set the title position
	 *
	 */
	private void setTitlePosition() {
		if( "TOP".equalsIgnoreCase(titlePosition) )
			chart.getTitle().setPosition(RectangleEdge.TOP);
		else if( "BOTTOM".equalsIgnoreCase(titlePosition) )
			chart.getTitle().setPosition(RectangleEdge.BOTTOM);
		else if( "LEFT".equalsIgnoreCase(titlePosition) )
			chart.getTitle().setPosition(RectangleEdge.LEFT);
		else if( "RIGHT".equalsIgnoreCase(titlePosition) )
			chart.getTitle().setPosition(RectangleEdge.RIGHT);
	}
	
	/**
	 * Set the subtitle position
	 *
	 */
	private void setSubtitlePosition() {
		if( "TOP".equalsIgnoreCase(subtitlePosition) )
			chart.getSubtitle(0).setPosition(RectangleEdge.TOP);
		else if( "BOTTOM".equalsIgnoreCase(subtitlePosition) )
			chart.getSubtitle(0).setPosition(RectangleEdge.BOTTOM);
		else if( "LEFT".equalsIgnoreCase(subtitlePosition) )
			chart.getSubtitle(0).setPosition(RectangleEdge.LEFT);
		else if( "RIGHT".equalsIgnoreCase(subtitlePosition) )
			chart.getSubtitle(0).setPosition(RectangleEdge.RIGHT);
	}

    /**
     * Release any acquired resources.
     */
	public void release() {
		super.release();
		title = null;
		subtitle = null;
		chartborder = false;
		usemap = false;
		antiAlias = false;
		bgColor = "white";
		createLegend = true;
		createTooltip = true;
		createUrl = false;
		titlePosition = Constant.DEFAULT_TITLE_POSITION;
		subtitlePosition = Constant.DEFAULT_SUBTITLE_POSITION;
		chartWidth = Constant.DEFAULT_CHART_WIDTH;
		chartHight = Constant.DEFAULT_CHART_HIGHT;
	}
	
	/**
	 * Return an iterator for the option labels or values, based on our
	 * configured properties.
	 *
	 * @param name Name of the bean attribute (if any)
	 * @param property Name of the bean property (if any)
	 *
	 * @exception JspException if an error occurs
	 */
	protected Iterator getIterator(String name, String property) throws JspException {
		
		// Identify the bean containing our collection
		String beanName = name;
		if (beanName == null) {
			throw new JspException(messages.getMessage("getter.bean", beanName));
		}
		
		Object bean = TagUtils.getInstance().lookup(pageContext, beanName, null);
				
		// Identify the collection itself
		Object collection = bean;
		if (property != null) {
			try {
				collection = BeanUtils.getProperty(bean, property);
				if (collection == null) {
					throw new JspException(messages.getMessage("getter.property", property));
				}
			} catch (IllegalAccessException e) {
				throw new JspException(messages.getMessage("getter.access", property, name));
			} catch (InvocationTargetException e) {
				Throwable t = e.getTargetException();
				throw new JspException(
						messages.getMessage("getter.result", property, t.toString()));
			} catch (NoSuchMethodException e) {
				throw new JspException(messages.getMessage("getter.method", property, name));
			}
		}
		
		// Construct and return an appropriate iterator
		if (collection.getClass().isArray()) {
			collection = Arrays.asList((Object[]) collection);
		}
		
		if (collection instanceof Collection) {
			return (((Collection) collection).iterator());
			
		} else if (collection instanceof Iterator) {
			return ((Iterator) collection);
			
		} else if (collection instanceof Map) {
			return (((Map) collection).entrySet().iterator());
			
		} else if (collection instanceof Enumeration) {
			return new IteratorAdapter((Enumeration) collection);
			
		} else {
			throw new JspException(
					messages.getMessage("baseChartTag.iterator", collection.toString()));
		}
	}
	
	/**
	 * <p>Geet a <code>java.awt.Color</code> using the input color,
	 *  the color can by in hexadecimal notation #FFFFFF or some a predefine color</p>
	 *  <br>
	 *  Predefined colors
	 *  <ul>
	 *  <li>black
	 *  <li>blue
	 *  <li>cyan
	 *  <li>darkgray
	 *  <li>green
	 *  <li>grey
	 *  <li>lightgray
	 *  <li>magenta
	 *  <li>orange
	 *  <li>pink
	 *  <li>red
	 *  <li>yellow
	 *  <li>white
	 *  </ul>
	 * @param color HTML colors (#FFFFFF format) or a predefined color
	 * @return a <code>java.awt.Color</code>
	 * @throws JspException if the <code>color</code> is <code>null</code>
	 * 	or the html format is not #ffffff
	 */
	private Color getColorByString(String color) throws JspException {
		// HTML colors (#FFFFFF format)
		if (color.startsWith("#")) {
			try {
				return new Color(Integer.parseInt(color.substring(1), 16));
			} catch (NumberFormatException e) {
				throw new JspException(messages.getMessage("color.format", color),e );
			} catch (NullPointerException e) {
				throw new JspException(messages.getMessage("item.null", color),e );
			}
		} else {
			// Colors by name
			if (color.equalsIgnoreCase("black"))
				return Color.black;
			if (color.equalsIgnoreCase("blue"))
				return Color.blue;
			if (color.equalsIgnoreCase("cyan"))
				return Color.cyan;
			if (color.equalsIgnoreCase("darkgray"))
				return Color.darkGray;
			if (color.equalsIgnoreCase("green"))
				return Color.green;
			if (color.equalsIgnoreCase("grey"))
				return Color.gray;
			if (color.equalsIgnoreCase("lightgray"))
				return Color.lightGray;
			if (color.equalsIgnoreCase("magenta"))
				return Color.magenta;
			if (color.equalsIgnoreCase("orange"))
				return Color.orange;
			if (color.equalsIgnoreCase("pink"))
				return Color.pink;
			if (color.equalsIgnoreCase("red"))
				return Color.red;
			if (color.equalsIgnoreCase("yellow"))
				return Color.yellow;
			// default color is white
			return Color.white;
		}
	}

	public String getSubtitle() {
		return subtitle;
	}
	public void setSubtitle(String subTitle) {
		this.subtitle = subTitle;
	}
	public String getTitle() {
		return title;
	}
	public boolean getChartborder() {
		return chartborder;
	}
	public void setChartborder(boolean chartborder) {
		this.chartborder = chartborder;
	}
	public String getSrc() {
		return src;
	}
	public void setSrc(String src) {
		this.src = src;
	}
	public void setTitle(String title) {
		this.title = title;
	}
	public boolean getUsemap() {
		return usemap;
	}
	public void setUsemap(boolean usemap) {
		this.usemap = usemap;
	}
	public boolean getAntiAlias() {
		return antiAlias;
	}
	public void setAntiAlias(boolean antiAlias) {
		this.antiAlias = antiAlias;
	}
	public JFreeChart getChart() {
		return chart;
	}
	public void setChart(JFreeChart chart) {
		this.chart = chart;
	}
	public String getTitlePosition() {
		return titlePosition;
	}
	public void setTitlePosition(String position) {
		this.titlePosition = position;
	}
	public String getSubtitlePosition() {
		return subtitlePosition;
	}
	public void setSubtitlePosition(String subtitlePosition) {
		this.subtitlePosition = subtitlePosition;
	}
	public float getFgAlpha() {
		return fgAlpha;
	}

	public void setFgAlpha(float fgAlpha) {
		this.fgAlpha = fgAlpha;
	}

	public float getBgAlpha() {
		return bgAlpha;
	}

	public void setBgAlpha(float bgAlpha) {
		this.bgAlpha = bgAlpha;
	}

	public String getBgColor() {
		return bgColor;
	}

	public void setBgColor(String bgColor) {
		this.bgColor = bgColor;
	}

	public boolean getCreateLegend() {
		return createLegend;
	}

	public void setCreateLegend(boolean createLegend) {
		this.createLegend = createLegend;
	}

	public boolean isCreateTooltip() {
		return createTooltip;
	}

	public void setCreateTooltip(boolean createToolTip) {
		this.createTooltip = createToolTip;
	}

	public boolean getCreateUrl() {
		return createUrl;
	}

	public void setCreateUrl(boolean createUrl) {
		this.createUrl = createUrl;
	}

	public int getChartHight() {
		return chartHight;
	}

	public void setChartHight(int chartHight) {
		this.chartHight = chartHight;
	}

	public int getChartWidth() {
		return chartWidth;
	}

	public void setChartWidth(int chartWidth) {
		this.chartWidth = chartWidth;
	}

	
}
