/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

package org.sf.charttaglibs;

import java.lang.reflect.InvocationTargetException;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.BodyTagSupport;

import org.apache.commons.beanutils.BeanUtils;

import org.sf.charttaglibs.utils.Message;
import org.sf.charttaglibs.utils.TagUtils;


/**
 * Base class for tags that render form elements capable of including JavaScript
 * event handlers and the common html tags. This class does not implement
 * the doStartTag() or doEndTag() methods. Subclasses should provide
 * appropriate implementations of these.
 *
 */
public abstract class BaseHandlerTag extends BodyTagSupport {

    /**
     * The message resources for this package.
     */
    protected static Message messages = new Message();
    

    /** The alternate text of this element. */
    private String alt = null;

    /**
     * The property to specify where to align the image.
     */
    protected String align = null;

    /**
     * The border size around the image.
     */
    protected String border = null;

    /**
     * The image height.
     */
    protected String height = null;

    /**
     * The horizontal spacing around the image.
     */
    protected String hspace = null;

    /**
     * Server-side image map declaration.
     */
    protected String ismap = null;

     /**
     * The JSP bean name for query parameters.
     */
    protected String name = null;

    /**
     * The vertical spacing around the image.
     */
    protected String vspace = null;

    /**
     * The image width.
     */
    protected String width = null;

    //  Mouse Events

    /** Mouse click event. */
    private String onclick = null;

    /** Component lost focus event. */
    private String onblur = null;

    /** Component has received focus event. */
    private String onfocus = null;
    
    /** Component has received abort event. */
    private String onabort = null;
    
    /** Component has received error event. */
    private String onerror = null;
    
    /** Component has received load event. */
    private String onload = null;

    
    private boolean xhtml = false;
    
    // --------------------------------------------------------- Public Methods

    /**
     * Release any acquired resources.
     */
    public void release() {
        super.release();
        alt = null;
        align = null;
        border = null;
        height = null;
        hspace = null;
        ismap = null;
        name = null;
        vspace = null;
        width = null;
        onclick = null;
        onblur = null;
        onfocus = null;
        onabort = null;
        onerror = null;
        onload = null;
    }

    // ------------------------------------------------------ Protected Methods

    /**
     * Prepares the html attributes for inclusion in the component's HTML tag.
     * @return The prepared String for inclusion in the HTML tag.
     * @exception JspException if invalid attributes are specified
     */
    protected String prepareHtmlElements() throws JspException {
    	
    	StringBuffer elements = new StringBuffer();
    	    	
    	prepareAttribute(elements , "alt", getAlt());
    	prepareAttribute(elements , "align", getAlign());
    	prepareAttribute(elements , "border", getBorder());
    	prepareAttribute(elements , "height", getHeight());
    	prepareAttribute(elements , "hspace", getHspace());
    	prepareAttribute(elements , "ismap", getIsmap());
    	prepareAttribute(elements , "name", getName());
    	prepareAttribute(elements , "vspace", getVspace());
    	prepareAttribute(elements , "width", getWidth());
    	
    	return elements.toString();

    }

    /**
     * Prepares the JavaScript event for inclusion in the component's HTML tag.
     * @return The prepared String for inclusion in the HTML tag.
     */
    protected String prepareJavaScriptsEvent() {
        StringBuffer event = new StringBuffer();
    	prepareAttribute(event, "onclick", getOnclick());
        prepareAttribute(event, "onblur", getOnblur());
        prepareAttribute(event, "onfocus", getOnfocus());
    	prepareAttribute(event, "onabort", getOnabort());
        prepareAttribute(event, "onerror", getOnerror());
        prepareAttribute(event, "onload", getOnload());
        return event.toString();
    }
    
    /**
     * 'Hook' to enable tags to be extended and 
     *  additional attributes added.
     * @param handlers The StringBuffer that output will be appended to.
     */
    protected void prepareOtherAttributes(StringBuffer handlers) {
    }

    /**
     * Prepares an attribute if the value is not null, appending it to the the given
     * StringBuffer.
     * @param handlers The StringBuffer that output will be appended to.
     */
    protected void prepareAttribute(StringBuffer handlers, String name, Object value) {
        if (value != null) {
            handlers.append(" ");
            handlers.append(name);
            handlers.append("=\"");
            handlers.append(value);
            handlers.append("\"");
        }
    }

    /**
     * Allows HTML tags to find out if they're nested within an %lt;html:html&gt; tag that
     * has xhtml set to true.
     * @return true if the tag is nested within an html tag with xhtml set to true, false
     * otherwise.
     */
    protected boolean isXhtml() {
        return xhtml;
    }

    /**
     * Returns the closing brace for an input element depending on xhtml status.  The tag
     * must be nested within an %lt;html:html&gt; tag that has xhtml set to true.
     * @return String - &gt; if xhtml is false, /&gt; if xhtml is true
     */
    protected String getElementClose() {
        return this.isXhtml() ? " />" : ">";
    }

    /**
     * Searches all scopes for the bean and calls BeanUtils.getProperty() with the 
     * given arguments and converts any exceptions into JspException.
     * 
     * @param beanName The name of the object to get the property from.
     * @param property The name of the property to get.
     * @return The value of the property.
     * @throws JspException
     */
    protected String lookupProperty(String beanName, String property)
        throws JspException {

        Object bean = TagUtils.getInstance().lookup(this.pageContext, beanName, null);
        if (bean == null) {
            throw new JspException(messages.getMessage("getter.bean", beanName));
        }

        try {
            return BeanUtils.getProperty(bean, property);

        } catch (IllegalAccessException e) {
            throw new JspException(
                messages.getMessage("getter.access", property, beanName));

        } catch (InvocationTargetException e) {
            Throwable t = e.getTargetException();
            throw new JspException(
                messages.getMessage("getter.result", property, t.toString()));

        } catch (NoSuchMethodException e) {
            throw new JspException(
                messages.getMessage("getter.method", property, beanName));
        }
    }

	public String getAlign() {
		return align;
	}

	public void setAlign(String align) {
		this.align = align;
	}

	public String getAlt() {
		return alt;
	}

	public void setAlt(String alt) {
		this.alt = alt;
	}

	public String getBorder() {
		return border;
	}

	public void setBorder(String border) {
		this.border = border;
	}

	public String getHeight() {
		return height;
	}

	public void setHeight(String height) {
		this.height = height;
	}

	public String getHspace() {
		return hspace;
	}

	public void setHspace(String hspace) {
		this.hspace = hspace;
	}

	public String getIsmap() {
		return ismap;
	}

	public void setIsmap(String ismap) {
		this.ismap = ismap;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getOnabort() {
		return onabort;
	}

	public void setOnabort(String onabort) {
		this.onabort = onabort;
	}

	public String getOnblur() {
		return onblur;
	}

	public void setOnblur(String onblur) {
		this.onblur = onblur;
	}

	public String getOnclick() {
		return onclick;
	}

	public void setOnclick(String onclick) {
		this.onclick = onclick;
	}

	public String getOnerror() {
		return onerror;
	}

	public void setOnerror(String onerror) {
		this.onerror = onerror;
	}

	public String getOnfocus() {
		return onfocus;
	}

	public void setOnfocus(String onfocus) {
		this.onfocus = onfocus;
	}

	public String getOnload() {
		return onload;
	}

	public void setOnload(String onload) {
		this.onload = onload;
	}

	public String getVspace() {
		return vspace;
	}

	public void setVspace(String vspace) {
		this.vspace = vspace;
	}

	public String getWidth() {
		return width;
	}

	public void setWidth(String width) {
		this.width = width;
	}

}
